/* iRiver iFP 7XX Emulator
   Copyright (C) 2005 Tomasz Malesinski <tmal@mimuw.edu.pl>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef _CPU_H_
#define _CPU_H_

#include "emuriver.h"

#define CPU_STOP_NORMAL 1
#define CPU_STOP_BREAK 2
#define CPU_STOP_RWATCH 3
#define CPU_STOP_WWATCH 4
#define CPU_STOP_AWATCH 5
#define CPU_STOP_TERMINATE 6
#define CPU_STOP_QUIT 7
#define CPU_STOP_ILLEGAL 8

#define CPU_HISTORY_SIZE 256

struct breaks {
  int n, max_n;
  uint32 *tab;
};

struct cpu {
  uint32 reg[16];
  uint32 cpsr;
  uint32 spsr;
  uint32 saved_svc[3];
  uint32 saved_abt[3];
  uint32 saved_und[3];
  uint32 saved_irq[3];
  uint32 saved_fiq[8];
  struct memory *mem;
  struct breaks breakpoints;
  struct breaks rwatchpoints;
  struct breaks wwatchpoints;
  struct breaks awatchpoints;
  int stop;
  uint32 break_addr;

  int (*irq_p)(void *data);
  void *irq_data;

  uint32 history[CPU_HISTORY_SIZE];
  int history_ptr;

  struct profile *profile;
};

struct cpu *create_cpu(struct memory *memory, struct profile *profile);
int cpu_go(struct cpu *cpu, int n_instr);
void cpu_set_irq_p(struct cpu *cpu, int (*irq_p)(void *),
		   void *irq_data);

#define CPU_BRK_BREAK 1
#define CPU_BRK_RWATCH 2
#define CPU_BRK_WWATCH 3
#define CPU_BRK_AWATCH 4

void cpu_remove_break(struct cpu *cpu, int type, uint32 val);
int cpu_insert_break(struct cpu *cpu, int type, uint32 val);

void cpu_history_to_string(struct cpu *cpu, char *buf);

#endif
