#include <stdio.h>
#include <stdlib.h>
#include "flashimg.h"
#include "flashimgtool.h"
#include "common.h"

struct flashimg *flashimg_create(int model, char *fname, int offset) {
  struct flashimg *flash;

  flash = xmalloc(sizeof(struct flashimg));
  if (!flash)
    return NULL;

  flash->model = model;
  flash->fp = fopen(fname, "r+b");
  if (!flash->fp) {
    error_sys("%s: ", fname);
    free(flash);
    return NULL;
  }
  return flash;
}

void flashimg_free(struct flashimg *flash) {
  if (flash) {
    if (flash->fp)
      fclose(flash->fp);
    free(flash);
  }
}

static int readwrite_fragment(FILE *fp, size_t offset,
                              size_t fstart, size_t  flength,
                              char *buf, size_t bstart, size_t blength,
                              int write_op) {
    size_t foffset, boffset;
    size_t length;

    if (fstart > bstart) {
        foffset = 0;
        boffset = fstart - bstart;
    } else {
        foffset = bstart - fstart;
        boffset = 0;
    }
    if (flength <= foffset || blength <= boffset)
        return 0;
    length = MIN(flength - boffset, blength - boffset);

    if (length > 0) {
        int res;
        if (fseek(fp, offset + foffset, SEEK_SET) == -1)
            return -1;
        if (write_op)
            res = fwrite(buf + boffset, 1, length, fp);
        else
            res = fread(buf + boffset, 1, length, fp);
        if (res != length)
            return -1;
    }
    return 0;
}

int flashimg_write(struct flashimg *flash, const char *buf,
                   int page, int start, int length) {
    if (readwrite_fragment(flash->fp, flash->offset + page * 64,
                           2048, 64,
                           (char *)buf, start, length, 1))
        return -1;
    if (readwrite_fragment(flash->fp,
                           flash->offset + page * 2048
                           + flashimg_n_pages(flash) * 64,
                           0, 2048,
                           (char *)buf, start, length, 1))
        return -1;
    return 0;
}

int flashimg_read(struct flashimg *flash, char *buf,
                  int page, int start, int length) {
    if (readwrite_fragment(flash->fp, flash->offset + page * 64,
                           2048, 64,
                           buf, start, length, 0))
        return -1;
    if (readwrite_fragment(flash->fp,
                           flash->offset + page * 2048
                           + flashimg_n_pages(flash) * 64,
                           0, 2048,
                           buf, start, length, 0))
        return -1;
    return 0;
}

int flashimg_n_pages(struct flashimg *flash) {
  switch (flash->model) {
  case FLASHIMG_256_8:
    return 128 * 1024;
  case FLASHIMG_512_8:
    return 256 * 1024;
  case FLASHIMG_1024_8:
    return 512 * 1024;
  default:
    return 0;
  }
}
