/* iRiver iFP 7XX Emulator
   Copyright (C) 2006 Tomasz Malesinski <tmal@mimuw.edu.pl>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include <stdio.h>
#include "profile.h"

static char *counter_name[PROFILE_NUM_COUNTERS] = {
  "CPUC", "IRMA", "SRMA", "CMIS", "CHIT", "CFLU"
};

struct profile *create_profile(void) {
  struct profile *profile;
  int i;
  
  profile = xmalloc(sizeof(struct profile));
  if (!profile) return NULL;

  for (i = 0; i < 1 << 16; i++)
    profile->region[i] = 0;
  
  return profile;
}

static struct profile_region *get_region(struct profile *profile,
					 int region) {
  if (!profile->region[region]) {
    profile->region[region] = xmalloc(sizeof(struct profile_region));
    if (profile->region[region]) {
      int i;
      for (i = 0; i < PROFILE_NUM_COUNTERS; i++)
	profile->region[region]->counter[i] = NULL;
    }
  }
  return profile->region[region];
}

static int create_counter(struct profile_region *profile_region,
			  int counter) {
  if (profile_region->counter[counter])
    return 0;
  profile_region->counter[counter] = xmalloc(sizeof(int) * (1 << 16));
  if (profile_region->counter[counter]) {
    memset(profile_region->counter[counter], 0, sizeof(int) * (1 << 16));
    return 0;
  } else
    return -1;
}

int profile_add_counter(struct profile *profile, int counter,
			uint32 start, uint32 end) {
  int i;
  for (i = start >> 16; i <= end >> 16; i++) {
    struct profile_region *region = get_region(profile, i);
    if (!region) return -1;
    if (create_counter(region, counter) < 0)
      return -1;
  }
  return 0;
}

void profile_count(struct profile *profile, uint32 addr,
		   int counter, int value) {
  if (profile) {
    struct profile_region *region = profile->region[addr >> 16];
    if (region && region->counter[counter])
      region->counter[counter][addr & ((1 << 16) - 1)] += value;
  }
}

void profile_dump(struct profile *profile, FILE *fp) {
  int r, c, i;
  for (r = 0; r < 1 << 16; r++)
    if (profile->region[r])
      for (i = 0; i < 1 << 16; i++) {
	int non_zero = 0;
	for (c = 0; c < PROFILE_NUM_COUNTERS && !non_zero; c++)
	  if (profile->region[r]->counter[c] &&
	      profile->region[r]->counter[c][i])
	    non_zero = 1;
	if (!non_zero) continue;
	fprintf(fp, "%08x:", (r << 16) + i);
	for (c = 0; c < PROFILE_NUM_COUNTERS; c++)
	  if (profile->region[r]->counter[c])
	    fprintf(fp, " %s=%9d", counter_name[c],
		    profile->region[r]->counter[c][i]);
	fprintf(fp, "\n");
      }
}
