/* iRiver iFP 7XX Emulator
   Copyright (C) 2005 Tomasz Malesinski <tmal@mimuw.edu.pl>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "SDL.h"
#include "log.h"
#include "lcd.h"
#include "ui.h"

static int sdl_after_init = 0;
static SDL_Surface *screen = NULL;
static char key_pressed[10];

struct key {
  int sdl_key;
  int iriver_key;
};

static struct key keys[] = {
  {SDLK_q, 0},
  {SDLK_1, 1},
  {SDLK_2, 2},
  {SDLK_3, 3},
  {SDLK_SPACE, 4},
  {SDLK_j, 5},
  {SDLK_LEFT, 5},
  {SDLK_l, 6},
  {SDLK_RIGHT, 6},
  {SDLK_i, 7},
  {SDLK_UP, 7},
  {SDLK_k, 8},
  {SDLK_DOWN, 8},
  {-1, -1}
};

void sdl_error(char *msg) {
  logmsg(LOG_GENERAL, 1, "SDL: %s: ", msg, SDL_GetError());
}

static int uisdl_init(void) {
  int i;
  SDL_Color col[2];

  if (SDL_Init(SDL_INIT_VIDEO) < 0) {
    sdl_error("SDL_Init");
    return -1;
  }
  sdl_after_init = 1;

  screen = SDL_SetVideoMode(LCD_WIDTH, LCD_HEIGHT, 8, SDL_SWSURFACE);
  if (!screen) {
    sdl_error("SDL_SetVideoMode");
    return -1;
  }

  col[0].r = 128;
  col[0].g = 128;
  col[0].b = 128;
  col[1].r = 10;
  col[1].g = 10;
  col[1].b = 10;
  SDL_SetColors(screen, col, 0, 2);

  for (i = 0; i < 10; i++)
    key_pressed[i] = 0;

  return 0;
}

static void uisdl_cleanup(void) {
  if (sdl_after_init)
    SDL_Quit();
}


static int key_no(int sym) {
  int i;
  for (i = 0; keys[i].sdl_key >= 0; i++)
    if (sym == keys[i].sdl_key)
      return keys[i].iriver_key;
  return -1;
}

static void events_handle(void) {
  SDL_Event ev;
  int k;

  while (SDL_PollEvent(&ev)) {
    switch (ev.type) {
    case SDL_QUIT:
      key_pressed[0] = 1;
      break;
    case SDL_KEYDOWN:
      k = key_no(ev.key.keysym.sym);
      if (k >= 0)
	key_pressed[k] = 1;
      else if (ev.key.keysym.sym == SDLK_h)
	key_pressed[9] = !key_pressed[9];
      break;
    case SDL_KEYUP:
      k = key_no(ev.key.keysym.sym);
      if (k >= 0)
	key_pressed[k] = 0;
      break;
    }
  }
}

static void uisdl_refresh(struct lcd *lcd) {
  int i, w, h, x, y;
  Uint8 *p;

  SDL_LockSurface(screen);
  for (y = 0; y < LCD_HEIGHT; y++) {
    p = (Uint8 *)screen->pixels + screen->pitch * y;
    for (x = 0; x < LCD_WIDTH; x++)
      *p++ = lcd_get_pixel(lcd, x, y) ? 1 : 0;
  }
  SDL_UnlockSurface(screen);
  SDL_UpdateRect(screen, 0, 0, 0, 0);

  events_handle();
}

static int uisdl_get_key(int key) {
  return key_pressed[key];
}

static void uisdl_set_status(char *s) {
}

struct ui uisdl_functions = {
  uisdl_init,
  uisdl_cleanup,
  uisdl_refresh,
  uisdl_get_key,
  uisdl_set_status
};
